@@ -1,20 +1,20 @@
-# Construct a Booster from cachelist
+# Construct an internal xgboost Booster and return a handle to it.
 # internal utility function
-xgb.Booster <- function(params = list(), cachelist = list(), modelfile = NULL) {
+xgb.Booster.handle <- function(params = list(), cachelist = list(), modelfile = NULL) {
   if (typeof(cachelist) != "list" ||
-      any(sapply(cachelist, class) != 'xgb.DMatrix')) {
-    stop("xgb.Booster only accepts list of DMatrix as cachelist")
+      !all(vapply(cachelist, inherits, logical(1), what = 'xgb.DMatrix'))) {
+    stop("cachelist must be a list of xgb.DMatrix objects")
   }
 
-  handle <- .Call("XGBoosterCreate_R", cachelist, PACKAGE = "xgboost")
+  handle <- .Call(XGBoosterCreate_R, cachelist)
   if (!is.null(modelfile)) {
     if (typeof(modelfile) == "character") {
-      .Call("XGBoosterLoadModel_R", handle, modelfile, PACKAGE = "xgboost")
+      .Call(XGBoosterLoadModel_R, handle, modelfile[1])
     } else if (typeof(modelfile) == "raw") {
-      .Call("XGBoosterLoadModelFromRaw_R", handle, modelfile, PACKAGE = "xgboost")
-    } else if (class(modelfile) == "xgb.Booster") {
-      modelfile <- xgb.Booster.check(modelfile, saveraw=TRUE)
-      .Call("XGBoosterLoadModelFromRaw_R", handle, modelfile$raw, PACKAGE = "xgboost")
+      .Call(XGBoosterLoadModelFromRaw_R, handle, modelfile)
+    } else if (inherits(modelfile, "xgb.Booster")) {
+      bst <- xgb.Booster.complete(modelfile, saveraw = TRUE)
+      .Call(XGBoosterLoadModelFromRaw_R, handle, bst$raw)
     } else {
       stop("modelfile must be either character filename, or raw booster dump, or xgb.Booster object")
     }
@@ -34,6 +34,17 @@ xgb.handleToBooster <- function(handle, raw = NULL) {
   return(bst)
 }
 
+# Check whether xgb.Booster.handle is null
+# internal utility function
+is.null.handle <- function(handle) {
+  if (!identical(class(handle), "xgb.Booster.handle"))
+    stop("argument type must be xgb.Booster.handle")
+
+  if (is.null(handle) || .Call(XGCheckNullPtr_R, handle))
+    return(TRUE)
+  return(FALSE)
+}
+
 # Return a verified to be valid handle out of either xgb.Booster.handle or xgb.Booster
 # internal utility function
 xgb.get.handle <- function(object) {
@@ -42,94 +53,171 @@ xgb.get.handle <- function(object) {
     xgb.Booster.handle = object,
     stop("argument must be of either xgb.Booster or xgb.Booster.handle class")
   )
-  if (is.null(handle) || .Call("XGCheckNullPtr_R", handle, PACKAGE="xgboost")) {
+  if (is.null.handle(handle)) {
     stop("invalid xgb.Booster.handle")
   }
   handle
 }
 
-# Check whether an xgb.Booster object is complete
-# internal utility function
-xgb.Booster.check <- function(bst, saveraw = TRUE) {
-  if (class(bst) != "xgb.Booster")
+#' Restore missing parts of an incomplete xgb.Booster object.
+#'
+#' It attempts to complete an \code{xgb.Booster} object by restoring either its missing
+#' raw model memory dump (when it has no \code{raw} data but its \code{xgb.Booster.handle} is valid)
+#' or its missing internal handle (when its \code{xgb.Booster.handle} is not valid
+#' but it has a raw Booster memory dump).
+#'
+#' @param object object of class \code{xgb.Booster}
+#' @param saveraw a flag indicating whether to append \code{raw} Booster memory dump data
+#'                when it doesn't already exist.
+#'
+#' @details
+#'
+#' While this method is primarily for internal use, it might be useful in some practical situations.
+#'
+#' E.g., when an \code{xgb.Booster} model is saved as an R object and then is loaded as an R object,
+#' its handle (pointer) to an internal xgboost model would be invalid. The majority of xgboost methods
+#' should still work for such a model object since those methods would be using
+#' \code{xgb.Booster.complete} internally. However, one might find it to be more efficient to call the
+#' \code{xgb.Booster.complete} function explicitely once after loading a model as an R-object.
+#' That would prevent further repeated implicit reconstruction of an internal booster model.
+#'
+#' @return
+#' An object of \code{xgb.Booster} class.
+#'
+#' @examples
+#'
+#' data(agaricus.train, package='xgboost')
+#' bst <- xgboost(data = agaricus.train$data, label = agaricus.train$label, max_depth = 2,
+#'                eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
+#' saveRDS(bst, "xgb.model.rds")
+#'
+#' bst1 <- readRDS("xgb.model.rds")
+#' # the handle is invalid:
+#' print(bst1$handle)
+#'
+#' bst1 <- xgb.Booster.complete(bst1)
+#' # now the handle points to a valid internal booster model:
+#' print(bst1$handle)
+#'
+#' @export
+xgb.Booster.complete <- function(object, saveraw = TRUE) {
+  if (!inherits(object, "xgb.Booster"))
     stop("argument type must be xgb.Booster")
-  
-  isnull <- is.null(bst$handle)
-  if (!isnull) {
-    isnull <- .Call("XGCheckNullPtr_R", bst$handle, PACKAGE="xgboost")
-  }
-  if (isnull) {
-    bst$handle <- xgb.Booster(modelfile = bst$raw)
+
+  if (is.null.handle(object$handle)) {
+    object$handle <- xgb.Booster.handle(modelfile = object$raw)
   } else {
-    if (is.null(bst$raw) && saveraw)
-      bst$raw <- xgb.save.raw(bst$handle)
+    if (is.null(object$raw) && saveraw)
+      object$raw <- xgb.save.raw(object$handle)
   }
-  return(bst)
+  return(object)
 }
 
-
 #' Predict method for eXtreme Gradient Boosting model
-#' 
+#'
 #' Predicted values based on either xgboost model or model handle object.
-#' 
+#'
 #' @param object Object of class \code{xgb.Booster} or \code{xgb.Booster.handle}
 #' @param newdata takes \code{matrix}, \code{dgCMatrix}, local data file or \code{xgb.DMatrix}.
 #' @param missing Missing is only used when input is dense matrix. Pick a float value that represents
 #'        missing values in data (e.g., sometimes 0 or some other extreme value is used).
-#' @param outputmargin whether the prediction should be returned in the for of original untransformed 
-#'        sum of predictions from boosting iterations' results. E.g., setting \code{outputmargin=TRUE} for 
+#' @param outputmargin whether the prediction should be returned in the for of original untransformed
+#'        sum of predictions from boosting iterations' results. E.g., setting \code{outputmargin=TRUE} for
 #'        logistic regression would result in predictions for log-odds instead of probabilities.
 #' @param ntreelimit limit the number of model's trees or boosting iterations used in prediction (see Details).
 #'        It will use all the trees by default (\code{NULL} value).
-#' @param predleaf whether predict leaf index instead. 
-#' @param reshape whether to reshape the vector of predictions to a matrix form when there are several 
+#' @param predleaf whether predict leaf index instead.
+#' @param predcontrib whether to return feature contributions to individual predictions instead (see Details).
+#' @param approxcontrib whether to use a fast approximation for feature contributions (see Details).
+#' @param reshape whether to reshape the vector of predictions to a matrix form when there are several
 #'        prediction outputs per case. This option has no effect when \code{predleaf = TRUE}.
 #' @param ... Parameters passed to \code{predict.xgb.Booster}
-#' 
-#' @details  
+#'
+#' @details
 #' Note that \code{ntreelimit} is not necessarily equal to the number of boosting iterations
 #' and it is not necessarily equal to the number of trees in a model.
 #' E.g., in a random forest-like model, \code{ntreelimit} would limit the number of trees.
-#' But for multiclass classification, there are multiple trees per iteration, 
-#' but \code{ntreelimit} limits the number of boosting iterations.
-#' 
-#' Also note that \code{ntreelimit} would currently do nothing for predictions from gblinear, 
-#' since gblinear doesn't keep its boosting history. 
-#' 
-#' One possible practical applications of the \code{predleaf} option is to use the model 
-#' as a generator of new features which capture non-linearity and interactions, 
-#' e.g., as implemented in \code{\link{xgb.create.features}}. 
-#' 
-#' @return 
+#' But for multiclass classification, while there are multiple trees per iteration,
+#' \code{ntreelimit} limits the number of boosting iterations.
+#'
+#' Also note that \code{ntreelimit} would currently do nothing for predictions from gblinear,
+#' since gblinear doesn't keep its boosting history.
+#'
+#' One possible practical applications of the \code{predleaf} option is to use the model
+#' as a generator of new features which capture non-linearity and interactions,
+#' e.g., as implemented in \code{\link{xgb.create.features}}.
+#'
+#' Setting \code{predcontrib = TRUE} allows to calculate contributions of each feature to
+#' individual predictions. For "gblinear" booster, feature contributions are simply linear terms
+#' (feature_beta * feature_value). For "gbtree" booster, feature contributions are SHAP
+#' values (Lundberg 2017) that sum to the difference between the expected output
+#' of the model and the current prediction (where the hessian weights are used to compute the expectations).
+#' Setting \code{approxcontrib = TRUE} approximates these values following the idea explained
+#' in \url{http://blog.datadive.net/interpreting-random-forests/}.
+#'
+#' @return
 #' For regression or binary classification, it returns a vector of length \code{nrows(newdata)}.
-#' For multiclass classification, either a \code{num_class * nrows(newdata)} vector or 
-#' a \code{(nrows(newdata), num_class)} dimension matrix is returned, depending on 
+#' For multiclass classification, either a \code{num_class * nrows(newdata)} vector or
+#' a \code{(nrows(newdata), num_class)} dimension matrix is returned, depending on
 #' the \code{reshape} value.
-#' 
-#' When \code{predleaf = TRUE}, the output is a matrix object with the 
+#'
+#' When \code{predleaf = TRUE}, the output is a matrix object with the
 #' number of columns corresponding to the number of trees.
-#' 
+#'
+#' When \code{predcontrib = TRUE} and it is not a multiclass setting, the output is a matrix object with
+#' \code{num_features + 1} columns. The last "+ 1" column in a matrix corresponds to bias.
+#' For a multiclass case, a list of \code{num_class} elements is returned, where each element is
+#' such a matrix. The contribution values are on the scale of untransformed margin
+#' (e.g., for binary classification would mean that the contributions are log-odds deviations from bias).
+#'
 #' @seealso
 #' \code{\link{xgb.train}}.
 #' 
+#' @references
+#'
+#' Scott M. Lundberg, Su-In Lee, "A Unified Approach to Interpreting Model Predictions", NIPS Proceedings 2017, \url{https://arxiv.org/abs/1705.07874}
+#'
+#' Scott M. Lundberg, Su-In Lee, "Consistent feature attribution for tree ensembles", \url{https://arxiv.org/abs/1706.06060}
+#'
 #' @examples
 #' ## binary classification:
-#' 
+#'
 #' data(agaricus.train, package='xgboost')
 #' data(agaricus.test, package='xgboost')
 #' train <- agaricus.train
 #' test <- agaricus.test
-#' 
-#' bst <- xgboost(data = train$data, label = train$label, max_depth = 2, 
-#'                eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
+#'
+#' bst <- xgboost(data = train$data, label = train$label, max_depth = 2,
+#'                eta = 0.5, nthread = 2, nrounds = 5, objective = "binary:logistic")
 #' # use all trees by default
 #' pred <- predict(bst, test$data)
 #' # use only the 1st tree
-#' pred <- predict(bst, test$data, ntreelimit = 1)
-#' 
-#' 
+#' pred1 <- predict(bst, test$data, ntreelimit = 1)
+#'
+#' # Predicting tree leafs:
+#' # the result is an nsamples X ntrees matrix
+#' pred_leaf <- predict(bst, test$data, predleaf = TRUE)
+#' str(pred_leaf)
+#'
+#' # Predicting feature contributions to predictions:
+#' # the result is an nsamples X (nfeatures + 1) matrix
+#' pred_contr <- predict(bst, test$data, predcontrib = TRUE)
+#' str(pred_contr)
+#' # verify that contributions' sums are equal to log-odds of predictions (up to float precision):
+#' summary(rowSums(pred_contr) - qlogis(pred))
+#' # for the 1st record, let's inspect its features that had non-zero contribution to prediction:
+#' contr1 <- pred_contr[1,]
+#' contr1 <- contr1[-length(contr1)]    # drop BIAS
+#' contr1 <- contr1[contr1 != 0]        # drop non-contributing features
+#' contr1 <- contr1[order(abs(contr1))] # order by contribution magnitude
+#' old_mar <- par("mar")
+#' par(mar = old_mar + c(0,7,0,0))
+#' barplot(contr1, horiz = TRUE, las = 2, xlab = "contribution to prediction in log-odds")
+#' par(mar = old_mar)
+#'
+#'
 #' ## multiclass classification in iris dataset:
-#' 
+#'
 #' lb <- as.numeric(iris$Species) - 1
 #' num_class <- 3
 #' set.seed(11)
@@ -145,7 +233,7 @@ xgb.Booster.check <- function(bst, saveraw = TRUE) {
 #' pred_labels <- max.col(pred) - 1
 #' # the following should result in the same error as seen in the last iteration
 #' sum(pred_labels != lb)/length(lb)
-#' 
+#'
 #' # compare that to the predictions from softmax:
 #' set.seed(11)
 #' bst <- xgboost(data = as.matrix(iris[, -5]), label = lb,
@@ -154,14 +242,14 @@ xgb.Booster.check <- function(bst, saveraw = TRUE) {
 #' pred <- predict(bst, as.matrix(iris[, -5]))
 #' str(pred)
 #' all.equal(pred, pred_labels)
-#' # prediction from using only 5 iterations should result 
+#' # prediction from using only 5 iterations should result
 #' # in the same error as seen in iteration 5:
 #' pred5 <- predict(bst, as.matrix(iris[, -5]), ntreelimit=5)
 #' sum(pred5 != lb)/length(lb)
-#' 
-#' 
+#'
+#'
 #' ## random forest-like model of 25 trees for binary classification:
-#' 
+#'
 #' set.seed(11)
 #' bst <- xgboost(data = train$data, label = train$label, max_depth = 5,
 #'                nthread = 2, nrounds = 1, objective = "binary:logistic",
@@ -177,35 +265,57 @@ xgb.Booster.check <- function(bst, saveraw = TRUE) {
 #'
 #' @rdname predict.xgb.Booster
 #' @export
-predict.xgb.Booster <- function(object, newdata, missing = NA,
-    outputmargin = FALSE, ntreelimit = NULL, predleaf = FALSE, reshape = FALSE, ...) {
+predict.xgb.Booster <- function(object, newdata, missing = NA, outputmargin = FALSE, ntreelimit = NULL,
+                                predleaf = FALSE, predcontrib = FALSE, approxcontrib = FALSE, reshape = FALSE, ...) {
 
-  object <- xgb.Booster.check(object, saveraw = FALSE)
-  if (class(newdata) != "xgb.DMatrix")
+  object <- xgb.Booster.complete(object, saveraw = FALSE)
+  if (!inherits(newdata, "xgb.DMatrix"))
     newdata <- xgb.DMatrix(newdata, missing = missing)
+  if (!is.null(object[["feature_names"]]) &&
+      !is.null(colnames(newdata)) &&
+      !identical(object[["feature_names"]], colnames(newdata)))
+    stop("Feature names stored in `object` and `newdata` are different!")
   if (is.null(ntreelimit))
     ntreelimit <- NVL(object$best_ntreelimit, 0)
+  if (NVL(object$params[['booster']], '') == 'gblinear')
+    ntreelimit <- 0
   if (ntreelimit < 0)
     stop("ntreelimit cannot be negative")
-  
-  option <- 0L + 1L * as.logical(outputmargin) + 2L * as.logical(predleaf)
-  
-  ret <- .Call("XGBoosterPredict_R", object$handle, newdata, option[1],
-               as.integer(ntreelimit), PACKAGE = "xgboost")
-  
-  if (length(ret) %% nrow(newdata) != 0)
-    stop("prediction length ", length(ret)," is not multiple of nrows(newdata) ", nrow(newdata))
-  npred_per_case <- length(ret) / nrow(newdata)
 
-  if (predleaf){
-    len <- nrow(newdata)
-    ret <- if (length(ret) == len) {
+  option <- 0L + 1L * as.logical(outputmargin) + 2L * as.logical(predleaf) + 4L * as.logical(predcontrib) + 8L * as.logical(approxcontrib)
+
+  ret <- .Call(XGBoosterPredict_R, object$handle, newdata, option[1], as.integer(ntreelimit))
+
+  n_ret <- length(ret)
+  n_row <- nrow(newdata)
+  npred_per_case <- n_ret / n_row
+
+  if (n_ret %% n_row != 0)
+    stop("prediction length ", n_ret, " is not multiple of nrows(newdata) ", n_row)
+
+  if (predleaf) {
+    ret <- if (n_ret == n_row) {
       matrix(ret, ncol = 1)
     } else {
-      t(matrix(ret, ncol = len))
+      matrix(ret, nrow = n_row, byrow = TRUE)
+    }
+  } else if (predcontrib) {
+    n_col1 <- ncol(newdata) + 1
+    n_group <- npred_per_case / n_col1
+    dnames <- if (!is.null(colnames(newdata))) list(NULL, c(colnames(newdata), "BIAS")) else NULL
+    ret <- if (n_ret == n_row) {
+      matrix(ret, ncol = 1, dimnames = dnames)
+    } else if (n_group == 1) {
+      matrix(ret, nrow = n_row, byrow = TRUE, dimnames = dnames)
+    } else {
+      grp_mask <- rep(seq_len(n_col1), n_row) +
+        rep((seq_len(n_row) - 1) * n_col1 * n_group, each = n_col1)
+      lapply(seq_len(n_group), function(g) {
+        matrix(ret[grp_mask + n_col1 * (g - 1)], nrow = n_row, byrow = TRUE, dimnames = dnames)
+      })
     }
   } else if (reshape && npred_per_case > 1) {
-    ret <- matrix(ret, ncol = length(ret) / nrow(newdata), byrow = TRUE)
+    ret <- matrix(ret, nrow = n_row, byrow = TRUE)
   }
   return(ret)
 }
@@ -227,9 +337,9 @@ predict.xgb.Booster.handle <- function(object, ...) {
 #'
 #' @param object Object of class \code{xgb.Booster} or \code{xgb.Booster.handle}.
 #' @param name a non-empty character string specifying which attribute is to be accessed.
-#' @param value a value of an attribute for \code{xgb.attr<-}; for \code{xgb.attributes<-} 
-#'        it's a list (or an object coercible to a list) with the names of attributes to set 
-#'        and the elements corresponding to attribute values. 
+#' @param value a value of an attribute for \code{xgb.attr<-}; for \code{xgb.attributes<-}
+#'        it's a list (or an object coercible to a list) with the names of attributes to set
+#'        and the elements corresponding to attribute values.
 #'        Non-character values are converted to character.
 #'        When attribute value is not a scalar, only the first index is used.
 #'        Use \code{NULL} to remove an attribute.
@@ -238,32 +348,32 @@ predict.xgb.Booster.handle <- function(object, ...) {
 #' The primary purpose of xgboost model attributes is to store some meta-data about the model.
 #' Note that they are a separate concept from the object attributes in R.
 #' Specifically, they refer to key-value strings that can be attached to an xgboost model,
-#' stored together with the model's binary representation, and accessed later 
+#' stored together with the model's binary representation, and accessed later
 #' (from R or any other interface).
 #' In contrast, any R-attribute assigned to an R-object of \code{xgb.Booster} class
 #' would not be saved by \code{xgb.save} because an xgboost model is an external memory object
 #' and its serialization is handled externally.
-#' Also, setting an attribute that has the same name as one of xgboost's parameters wouldn't 
-#' change the value of that parameter for a model. 
+#' Also, setting an attribute that has the same name as one of xgboost's parameters wouldn't
+#' change the value of that parameter for a model.
 #' Use \code{\link{xgb.parameters<-}} to set or change model parameters.
-#' 
+#'
 #' The attribute setters would usually work more efficiently for \code{xgb.Booster.handle}
 #' than for \code{xgb.Booster}, since only just a handle (pointer) would need to be copied.
 #' That would only matter if attributes need to be set many times.
 #' Note, however, that when feeding a handle of an \code{xgb.Booster} object to the attribute setters,
-#' the raw model cache of an \code{xgb.Booster} object would not be automatically updated, 
+#' the raw model cache of an \code{xgb.Booster} object would not be automatically updated,
 #' and it would be user's responsibility to call \code{xgb.save.raw} to update it.
-#' 
-#' The \code{xgb.attributes<-} setter either updates the existing or adds one or several attributes, 
+#'
+#' The \code{xgb.attributes<-} setter either updates the existing or adds one or several attributes,
 #' but it doesn't delete the other existing attributes.
-#' 
+#'
 #' @return
-#' \code{xgb.attr} returns either a string value of an attribute 
+#' \code{xgb.attr} returns either a string value of an attribute
 #' or \code{NULL} if an attribute wasn't stored in a model.
-#' 
-#' \code{xgb.attributes} returns a list of all attribute stored in a model 
+#'
+#' \code{xgb.attributes} returns a list of all attribute stored in a model
 #' or \code{NULL} if a model has no stored attributes.
-#' 
+#'
 #' @examples
 #' data(agaricus.train, package='xgboost')
 #' train <- agaricus.train
@@ -279,19 +389,19 @@ predict.xgb.Booster.handle <- function(object, ...) {
 #' bst1 <- xgb.load('xgb.model')
 #' print(xgb.attr(bst1, "my_attribute"))
 #' print(xgb.attributes(bst1))
-#' 
+#'
 #' # deletion:
 #' xgb.attr(bst1, "my_attribute") <- NULL
 #' print(xgb.attributes(bst1))
 #' xgb.attributes(bst1) <- list(a = NULL, b = NULL)
 #' print(xgb.attributes(bst1))
-#' 
+#'
 #' @rdname xgb.attr
 #' @export
 xgb.attr <- function(object, name) {
   if (is.null(name) || nchar(as.character(name[1])) == 0) stop("invalid attribute name")
   handle <- xgb.get.handle(object)
-  .Call("XGBoosterGetAttr_R", handle, as.character(name[1]), PACKAGE="xgboost")
+  .Call(XGBoosterGetAttr_R, handle, as.character(name[1]))
 }
 
 #' @rdname xgb.attr
@@ -308,7 +418,7 @@ xgb.attr <- function(object, name) {
       value <- as.character(value[1])
     }
   }
-  .Call("XGBoosterSetAttr_R", handle, as.character(name[1]), value, PACKAGE="xgboost")
+  .Call(XGBoosterSetAttr_R, handle, as.character(name[1]), value)
   if (is(object, 'xgb.Booster') && !is.null(object$raw)) {
     object$raw <- xgb.save.raw(object$handle)
   }
@@ -319,10 +429,10 @@ xgb.attr <- function(object, name) {
 #' @export
 xgb.attributes <- function(object) {
   handle <- xgb.get.handle(object)
-  attr_names <- .Call("XGBoosterGetAttrNames_R", handle, PACKAGE="xgboost")
+  attr_names <- .Call(XGBoosterGetAttrNames_R, handle)
   if (is.null(attr_names)) return(NULL)
   res <- lapply(attr_names, function(x) {
-    .Call("XGBoosterGetAttr_R", handle, x, PACKAGE="xgboost")
+    .Call(XGBoosterGetAttr_R, handle, x)
   })
   names(res) <- attr_names
   res
@@ -347,7 +457,7 @@ xgb.attributes <- function(object) {
   })
   handle <- xgb.get.handle(object)
   for (i in seq_along(a)) {
-    .Call("XGBoosterSetAttr_R", handle, names(a[i]), a[[i]], PACKAGE="xgboost")
+    .Call(XGBoosterSetAttr_R, handle, names(a[i]), a[[i]])
   }
   if (is(object, 'xgb.Booster') && !is.null(object$raw)) {
     object$raw <- xgb.save.raw(object$handle)
@@ -366,7 +476,7 @@ xgb.attributes <- function(object) {
 #' @details
 #' Note that the setter would usually work more efficiently for \code{xgb.Booster.handle}
 #' than for \code{xgb.Booster}, since only just a handle would need to be copied.
-#' 
+#'
 #' @examples
 #' data(agaricus.train, package='xgboost')
 #' train <- agaricus.train
@@ -375,7 +485,7 @@ xgb.attributes <- function(object) {
 #'                eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
 #'
 #' xgb.parameters(bst) <- list(eta = 0.1)
-#' 
+#'
 #' @rdname xgb.parameters
 #' @export
 `xgb.parameters<-` <- function(object, value) {
@@ -388,7 +498,7 @@ xgb.attributes <- function(object) {
   p <- lapply(p, function(x) as.character(x)[1])
   handle <- xgb.get.handle(object)
   for (i in seq_along(p)) {
-    .Call("XGBoosterSetParam_R", handle, names(p[i]), p[[i]], PACKAGE = "xgboost")
+    .Call(XGBoosterSetParam_R, handle, names(p[i]), p[[i]])
   }
   if (is(object, 'xgb.Booster') && !is.null(object$raw)) {
     object$raw <- xgb.save.raw(object$handle)
@@ -396,8 +506,8 @@ xgb.attributes <- function(object) {
   object
 }
 
-# Extract # of trees in a model
-# TODO: either add a getter to C-interface, or simply set an 'ntree' attribute after each iteration
+# Extract the number of trees in a model.
+# TODO: either add a getter to C-interface, or simply set an 'ntree' attribute after each iteration.
 # internal utility function
 xgb.ntree <- function(bst) {
   length(grep('^booster', xgb.dump(bst)))
@@ -405,36 +515,35 @@ xgb.ntree <- function(bst) {
 
 
 #' Print xgb.Booster
-#' 
+#'
 #' Print information about xgb.Booster.
-#' 
+#'
 #' @param x an xgb.Booster object
 #' @param verbose whether to print detailed data (e.g., attribute values)
 #' @param ... not currently used
-#' 
+#'
 #' @examples
 #' data(agaricus.train, package='xgboost')
 #' train <- agaricus.train
 #' bst <- xgboost(data = train$data, label = train$label, max_depth = 2,
 #'                eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
 #' attr(bst, 'myattr') <- 'memo'
-#' 
+#'
 #' print(bst)
 #' print(bst, verbose=TRUE)
 #'
-#' @method print xgb.Booster 
+#' @method print xgb.Booster
 #' @export
-print.xgb.Booster <- function(x, verbose=FALSE, ...) {
+print.xgb.Booster <- function(x, verbose = FALSE, ...) {
   cat('##### xgb.Booster\n')
-  
-  if (is.null(x$handle) || .Call("XGCheckNullPtr_R", x$handle, PACKAGE="xgboost")) {
-    cat("handle is invalid\n")
-    return(x)
-  }
-  
+
+  valid_handle <- is.null.handle(x$handle)
+  if (!valid_handle)
+    cat("Handle is invalid! Suggest using xgb.Booster.complete\n")
+
   cat('raw: ')
   if (!is.null(x$raw)) {
-    cat(format(object.size(x$raw), units="auto"), '\n')
+    cat(format(object.size(x$raw), units = "auto"), '\n')
   } else {
     cat('NULL\n')
   }
@@ -442,28 +551,30 @@ print.xgb.Booster <- function(x, verbose=FALSE, ...) {
     cat('call:\n  ')
     print(x$call)
   }
-  
+
   if (!is.null(x$params)) {
     cat('params (as set within xgb.train):\n')
-    cat( '  ', 
+    cat( '  ',
          paste(names(x$params),
                paste0('"', unlist(x$params), '"'),
-               sep=' = ', collapse=', '), '\n', sep='')
+               sep = ' = ', collapse = ', '), '\n', sep = '')
   }
   # TODO: need an interface to access all the xgboosts parameters
 
-  attrs <- xgb.attributes(x)
+  attrs <- character(0)
+  if (valid_handle)
+    attrs <- xgb.attributes(x)
   if (length(attrs) > 0) {
     cat('xgb.attributes:\n')
     if (verbose) {
       cat( paste(paste0('  ',names(attrs)),
                  paste0('"', unlist(attrs), '"'),
-                 sep=' = ', collapse='\n'), '\n', sep='')
+                 sep = ' = ', collapse = '\n'), '\n', sep = '')
     } else {
-      cat('  ', paste(names(attrs), collapse=', '), '\n', sep='')
+      cat('  ', paste(names(attrs), collapse = ', '), '\n', sep = '')
     }
   }
-  
+
   if (!is.null(x$callbacks) && length(x$callbacks) > 0) {
     cat('callbacks:\n')
     lapply(callback.calls(x$callbacks), function(x) {
@@ -471,24 +582,28 @@ print.xgb.Booster <- function(x, verbose=FALSE, ...) {
       print(x)
     })
   }
-  
-  cat('niter: ', x$niter, '\n', sep='')
+
+  if (!is.null(x$feature_names))
+    cat('# of features:', length(x$feature_names), '\n')
+
+  cat('niter: ', x$niter, '\n', sep = '')
   # TODO: uncomment when faster xgb.ntree is implemented
   #cat('ntree: ', xgb.ntree(x), '\n', sep='')
-  
-  for (n in setdiff(names(x), c('handle', 'raw', 'call', 'params', 'callbacks','evaluation_log','niter'))) {
+
+  for (n in setdiff(names(x), c('handle', 'raw', 'call', 'params', 'callbacks',
+                                'evaluation_log','niter','feature_names'))) {
     if (is.atomic(x[[n]])) {
-      cat(n, ': ', x[[n]], '\n', sep='')
+      cat(n, ':', x[[n]], '\n', sep = ' ')
     } else {
-      cat(n, ':\n\t', sep='')
+      cat(n, ':\n\t', sep = ' ')
       print(x[[n]])
     }
   }
-  
+
   if (!is.null(x$evaluation_log)) {
     cat('evaluation_log:\n')
     print(x$evaluation_log, row.names = FALSE, topn = 2)
   }
-  
+
   invisible(x)
 }
