@@ -4,19 +4,23 @@
  * \brief Implementation of learning algorithm.
  * \author Tianqi Chen
  */
-#include <xgboost/logging.h>
-#include <xgboost/learner.h>
 #include <dmlc/io.h>
+#include <dmlc/timer.h>
+#include <xgboost/learner.h>
+#include <xgboost/logging.h>
 #include <algorithm>
-#include <vector>
-#include <utility>
-#include <string>
-#include <sstream>
-#include <limits>
 #include <iomanip>
-#include "./common/io.h"
+#include <limits>
+#include <sstream>
+#include <string>
+#include <utility>
+#include <vector>
 #include "./common/common.h"
+#include "./common/host_device_vector.h"
+#include "./common/io.h"
 #include "./common/random.h"
+#include "common/timer.h"
+
 
 namespace xgboost {
 // implementation of base learner.
@@ -24,17 +28,14 @@ bool Learner::AllowLazyCheckPoint() const {
   return gbm_->AllowLazyCheckPoint();
 }
 
-std::vector<std::string>
-Learner::DumpModel(const FeatureMap& fmap,
-                   bool with_stats,
-                   std::string format) const {
+std::vector<std::string> Learner::DumpModel(const FeatureMap& fmap,
+                                            bool with_stats,
+                                            std::string format) const {
   return gbm_->DumpModel(fmap, with_stats, format);
 }
 
-
 /*! \brief training parameter for regression */
-struct LearnerModelParam
-    : public dmlc::Parameter<LearnerModelParam> {
+struct LearnerModelParam : public dmlc::Parameter<LearnerModelParam> {
   /* \brief global bias */
   bst_float base_score;
   /* \brief number of features  */
@@ -43,8 +44,10 @@ struct LearnerModelParam
   int num_class;
   /*! \brief Model contain additional properties */
   int contain_extra_attrs;
+  /*! \brief Model contain eval metrics */
+  int contain_eval_metrics;
   /*! \brief reserved field */
-  int reserved[30];
+  int reserved[29];
   /*! \brief constructor */
   LearnerModelParam() {
     std::memset(this, 0, sizeof(LearnerModelParam));
@@ -52,20 +55,21 @@ struct LearnerModelParam
   }
   // declare parameters
   DMLC_DECLARE_PARAMETER(LearnerModelParam) {
-    DMLC_DECLARE_FIELD(base_score).set_default(0.5f)
+    DMLC_DECLARE_FIELD(base_score)
+        .set_default(0.5f)
         .describe("Global bias of the model.");
-    DMLC_DECLARE_FIELD(num_feature).set_default(0)
-        .describe("Number of features in training data,"\
-                  " this parameter will be automatically detected by learner.");
-    DMLC_DECLARE_FIELD(num_class).set_default(0).set_lower_bound(0)
-        .describe("Number of class option for multi-class classifier. "\
-                  " By default equals 0 and corresponds to binary classifier.");
+    DMLC_DECLARE_FIELD(num_feature)
+        .set_default(0)
+        .describe(
+            "Number of features in training data,"
+            " this parameter will be automatically detected by learner.");
+    DMLC_DECLARE_FIELD(num_class).set_default(0).set_lower_bound(0).describe(
+        "Number of class option for multi-class classifier. "
+        " By default equals 0 and corresponds to binary classifier.");
   }
 };
 
-
-struct LearnerTrainParam
-    : public dmlc::Parameter<LearnerTrainParam> {
+struct LearnerTrainParam : public dmlc::Parameter<LearnerTrainParam> {
   // stored random seed
   int seed;
   // whether seed the PRNG each iteration
@@ -83,31 +87,48 @@ struct LearnerTrainParam
   // number of threads to use if OpenMP is enabled
   // if equals 0, use system default
   int nthread;
+  // flag to print out detailed breakdown of runtime
+  int debug_verbose;
   // declare parameters
   DMLC_DECLARE_PARAMETER(LearnerTrainParam) {
-    DMLC_DECLARE_FIELD(seed).set_default(0)
-        .describe("Random number seed during training.");
-    DMLC_DECLARE_FIELD(seed_per_iteration).set_default(false)
-        .describe("Seed PRNG determnisticly via iterator number, "\
-                  "this option will be switched on automatically on distributed mode.");
-    DMLC_DECLARE_FIELD(dsplit).set_default(0)
+    DMLC_DECLARE_FIELD(seed).set_default(0).describe(
+        "Random number seed during training.");
+    DMLC_DECLARE_FIELD(seed_per_iteration)
+        .set_default(false)
+        .describe(
+            "Seed PRNG determnisticly via iterator number, "
+            "this option will be switched on automatically on distributed "
+            "mode.");
+    DMLC_DECLARE_FIELD(dsplit)
+        .set_default(0)
         .add_enum("auto", 0)
         .add_enum("col", 1)
         .add_enum("row", 2)
         .describe("Data split mode for distributed training.");
-    DMLC_DECLARE_FIELD(tree_method).set_default(0)
+    DMLC_DECLARE_FIELD(tree_method)
+        .set_default(0)
         .add_enum("auto", 0)
         .add_enum("approx", 1)
         .add_enum("exact", 2)
+        .add_enum("hist", 3)
+        .add_enum("gpu_exact", 4)
+        .add_enum("gpu_hist", 5)
         .describe("Choice of tree construction method.");
-    DMLC_DECLARE_FIELD(test_flag).set_default("")
-        .describe("Internal test flag");
-    DMLC_DECLARE_FIELD(prob_buffer_row).set_default(1.0f).set_range(0.0f, 1.0f)
+    DMLC_DECLARE_FIELD(test_flag).set_default("").describe(
+        "Internal test flag");
+    DMLC_DECLARE_FIELD(prob_buffer_row)
+        .set_default(1.0f)
+        .set_range(0.0f, 1.0f)
         .describe("Maximum buffered row portion");
-    DMLC_DECLARE_FIELD(max_row_perbatch).set_default(std::numeric_limits<size_t>::max())
+    DMLC_DECLARE_FIELD(max_row_perbatch)
+        .set_default(std::numeric_limits<size_t>::max())
         .describe("maximum row per batch.");
-    DMLC_DECLARE_FIELD(nthread).set_default(0)
-        .describe("Number of threads to use.");
+    DMLC_DECLARE_FIELD(nthread).set_default(0).describe(
+        "Number of threads to use.");
+    DMLC_DECLARE_FIELD(debug_verbose)
+        .set_lower_bound(0)
+        .set_default(0)
+        .describe("flag to print out detailed breakdown of runtime");
   }
 };
 
@@ -115,43 +136,90 @@ DMLC_REGISTER_PARAMETER(LearnerModelParam);
 DMLC_REGISTER_PARAMETER(LearnerTrainParam);
 
 /*!
- * \brief learner that performs gradient boosting for a specific objective function.
- *  It does training and prediction.
+ * \brief learner that performs gradient boosting for a specific objective
+ * function. It does training and prediction.
  */
 class LearnerImpl : public Learner {
  public:
-  explicit LearnerImpl(const std::vector<std::shared_ptr<DMatrix> >& cache)
-      : cache_(cache) {
+  explicit LearnerImpl(std::vector<std::shared_ptr<DMatrix> >  cache)
+      : cache_(std::move(cache)) {
     // boosted tree
     name_obj_ = "reg:linear";
     name_gbm_ = "gbtree";
   }
 
-  void Configure(const std::vector<std::pair<std::string, std::string> >& args) override {
+  static void AssertGPUSupport() {
+#ifndef XGBOOST_USE_CUDA
+    LOG(FATAL) << "XGBoost version not compiled with GPU support.";
+#endif
+  }
+
+  void ConfigureUpdaters() {
+    if (tparam_.tree_method == 0 || tparam_.tree_method == 1 ||
+        tparam_.tree_method == 2) {
+      if (cfg_.count("updater") == 0) {
+        if (tparam_.dsplit == 1) {
+          cfg_["updater"] = "distcol";
+        } else if (tparam_.dsplit == 2) {
+          cfg_["updater"] = "grow_histmaker,prune";
+        }
+        if (tparam_.prob_buffer_row != 1.0f) {
+          cfg_["updater"] = "grow_histmaker,refresh,prune";
+        }
+      }
+    } else if (tparam_.tree_method == 3) {
+      /* histogram-based algorithm */
+      LOG(CONSOLE) << "Tree method is selected to be \'hist\', which uses a "
+                      "single updater "
+                   << "grow_fast_histmaker.";
+      cfg_["updater"] = "grow_fast_histmaker";
+    } else if (tparam_.tree_method == 4) {
+      this->AssertGPUSupport();
+      if (cfg_.count("updater") == 0) {
+        cfg_["updater"] = "grow_gpu,prune";
+      }
+      if (cfg_.count("predictor") == 0) {
+        cfg_["predictor"] = "gpu_predictor";
+      }
+    } else if (tparam_.tree_method == 5) {
+      this->AssertGPUSupport();
+      if (cfg_.count("updater") == 0) {
+        cfg_["updater"] = "grow_gpu_hist";
+      }
+      if (cfg_.count("predictor") == 0) {
+        cfg_["predictor"] = "gpu_predictor";
+      }
+    }
+  }
+
+  void Configure(
+      const std::vector<std::pair<std::string, std::string> >& args) override {
     // add to configurations
-    tparam.InitAllowUnknown(args);
+    tparam_.InitAllowUnknown(args);
+    monitor_.Init("Learner", tparam_.debug_verbose);
     cfg_.clear();
     for (const auto& kv : args) {
       if (kv.first == "eval_metric") {
         // check duplication
-        auto dup_check = [&kv](const std::unique_ptr<Metric>&m) {
+        auto dup_check = [&kv](const std::unique_ptr<Metric>& m) {
           return m->Name() != kv.second;
         };
         if (std::all_of(metrics_.begin(), metrics_.end(), dup_check)) {
           metrics_.emplace_back(Metric::Create(kv.second));
+          mparam_.contain_eval_metrics = 1;
         }
       } else {
         cfg_[kv.first] = kv.second;
       }
     }
-    if (tparam.nthread != 0) {
-      omp_set_num_threads(tparam.nthread);
+    if (tparam_.nthread != 0) {
+      omp_set_num_threads(tparam_.nthread);
     }
 
     // add additional parameters
     // These are cosntraints that need to be satisfied.
-    if (tparam.dsplit == 0 && rabit::IsDistributed()) {
-      tparam.dsplit = 2;
+    if (tparam_.dsplit == 0 && rabit::IsDistributed()) {
+      tparam_.dsplit = 2;
     }
 
     if (cfg_.count("num_class") != 0) {
@@ -161,22 +229,13 @@ class LearnerImpl : public Learner {
       }
     }
 
-    if (cfg_.count("max_delta_step") == 0 &&
-        cfg_.count("objective") != 0 &&
+    if (cfg_.count("max_delta_step") == 0 && cfg_.count("objective") != 0 &&
         cfg_["objective"] == "count:poisson") {
       cfg_["max_delta_step"] = "0.7";
     }
 
-    if (cfg_.count("updater") == 0) {
-      if (tparam.dsplit == 1) {
-        cfg_["updater"] = "distcol";
-      } else if (tparam.dsplit == 2) {
-        cfg_["updater"] = "grow_histmaker,prune";
-      }
-      if (tparam.prob_buffer_row != 1.0f) {
-        cfg_["updater"] = "grow_histmaker,refresh,prune";
-      }
-    }
+    ConfigureUpdaters();
+
     if (cfg_.count("objective") == 0) {
       cfg_["objective"] = "reg:linear";
     }
@@ -185,28 +244,26 @@ class LearnerImpl : public Learner {
     }
 
     if (!this->ModelInitialized()) {
-      mparam.InitAllowUnknown(args);
+      mparam_.InitAllowUnknown(args);
       name_obj_ = cfg_["objective"];
       name_gbm_ = cfg_["booster"];
       // set seed only before the model is initialized
-      common::GlobalRandom().seed(tparam.seed);
+      common::GlobalRandom().seed(tparam_.seed);
     }
 
     // set number of features correctly.
-    cfg_["num_feature"] = common::ToString(mparam.num_feature);
-    cfg_["num_class"] = common::ToString(mparam.num_class);
+    cfg_["num_feature"] = common::ToString(mparam_.num_feature);
+    cfg_["num_class"] = common::ToString(mparam_.num_class);
 
-    if (gbm_.get() != nullptr) {
+    if (gbm_ != nullptr) {
       gbm_->Configure(cfg_.begin(), cfg_.end());
     }
-    if (obj_.get() != nullptr) {
+    if (obj_ != nullptr) {
       obj_->Configure(cfg_.begin(), cfg_.end());
     }
   }
 
-  void InitModel() override {
-    this->LazyInitModel();
-  }
+  void InitModel() override { this->LazyInitModel(); }
 
   void Load(dmlc::Stream* fi) override {
     // TODO(tqchen) mark deprecation of old format.
@@ -218,13 +275,13 @@ class LearnerImpl : public Learner {
       CHECK_NE(header, "bs64")
           << "Base64 format is no longer supported in brick.";
       if (header == "binf") {
-        CHECK_EQ(fp.Read(&header[0], 4), 4);
+        CHECK_EQ(fp.Read(&header[0], 4), 4U);
       }
     }
     // use the peekable reader.
     fi = &fp;
     // read parameter
-    CHECK_EQ(fi->Read(&mparam, sizeof(mparam)), sizeof(mparam))
+    CHECK_EQ(fi->Read(&mparam_, sizeof(mparam_)), sizeof(mparam_))
         << "BoostLearner: wrong model format";
     {
       // backward compatibility code for compatible with old model type
@@ -240,77 +297,96 @@ class LearnerImpl : public Learner {
       if (len != 0) {
         name_obj_.resize(len);
         CHECK_EQ(fi->Read(&name_obj_[0], len), len)
-            <<"BoostLearner: wrong model format";
+            << "BoostLearner: wrong model format";
       }
     }
-    CHECK(fi->Read(&name_gbm_))
-        << "BoostLearner: wrong model format";
+    CHECK(fi->Read(&name_gbm_)) << "BoostLearner: wrong model format";
     // duplicated code with LazyInitModel
     obj_.reset(ObjFunction::Create(name_obj_));
-    gbm_.reset(GradientBooster::Create(name_gbm_, cache_, mparam.base_score));
+    gbm_.reset(GradientBooster::Create(name_gbm_, cache_, mparam_.base_score));
     gbm_->Load(fi);
-    if (mparam.contain_extra_attrs != 0) {
+    if (mparam_.contain_extra_attrs != 0) {
       std::vector<std::pair<std::string, std::string> > attr;
       fi->Read(&attr);
-      attributes_ = std::map<std::string, std::string>(
-          attr.begin(), attr.end());
+      attributes_ =
+          std::map<std::string, std::string>(attr.begin(), attr.end());
     }
     if (name_obj_ == "count:poisson") {
-        std::string max_delta_step;
-        fi->Read(&max_delta_step);
-        cfg_["max_delta_step"] = max_delta_step;
+      std::string max_delta_step;
+      fi->Read(&max_delta_step);
+      cfg_["max_delta_step"] = max_delta_step;
     }
-    cfg_["num_class"] = common::ToString(mparam.num_class);
-    cfg_["num_feature"] = common::ToString(mparam.num_feature);
+    if (mparam_.contain_eval_metrics != 0) {
+      std::vector<std::string> metr;
+      fi->Read(&metr);
+      for (auto name : metr) {
+        metrics_.emplace_back(Metric::Create(name));
+      }
+    }
+    cfg_["num_class"] = common::ToString(mparam_.num_class);
+    cfg_["num_feature"] = common::ToString(mparam_.num_feature);
     obj_->Configure(cfg_.begin(), cfg_.end());
   }
 
   // rabit save model to rabit checkpoint
-  void Save(dmlc::Stream *fo) const override {
-    fo->Write(&mparam, sizeof(LearnerModelParam));
+  void Save(dmlc::Stream* fo) const override {
+    fo->Write(&mparam_, sizeof(LearnerModelParam));
     fo->Write(name_obj_);
     fo->Write(name_gbm_);
     gbm_->Save(fo);
-    if (mparam.contain_extra_attrs != 0) {
+    if (mparam_.contain_extra_attrs != 0) {
       std::vector<std::pair<std::string, std::string> > attr(
           attributes_.begin(), attributes_.end());
       fo->Write(attr);
     }
     if (name_obj_ == "count:poisson") {
-        std::map<std::string, std::string>::const_iterator it = cfg_.find("max_delta_step");
-        if (it != cfg_.end())
-            fo->Write(it->second);
+      auto it =
+          cfg_.find("max_delta_step");
+      if (it != cfg_.end()) fo->Write(it->second);
+    }
+    if (mparam_.contain_eval_metrics != 0) {
+      std::vector<std::string> metr;
+      for (auto& ev : metrics_) {
+        metr.emplace_back(ev->Name());
+      }
+      fo->Write(metr);
     }
   }
 
   void UpdateOneIter(int iter, DMatrix* train) override {
+    monitor_.Start("UpdateOneIter");
     CHECK(ModelInitialized())
         << "Always call InitModel or LoadModel before update";
-    if (tparam.seed_per_iteration || rabit::IsDistributed()) {
-      common::GlobalRandom().seed(tparam.seed * kRandSeedMagic + iter);
+    if (tparam_.seed_per_iteration || rabit::IsDistributed()) {
+      common::GlobalRandom().seed(tparam_.seed * kRandSeedMagic + iter);
     }
     this->LazyInitDMatrix(train);
+    monitor_.Start("PredictRaw");
     this->PredictRaw(train, &preds_);
-    obj_->GetGradient(preds_, train->info(), iter, &gpair_);
+    monitor_.Stop("PredictRaw");
+    monitor_.Start("GetGradient");
+    obj_->GetGradient(&preds_, train->Info(), iter, &gpair_);
+    monitor_.Stop("GetGradient");
     gbm_->DoBoost(train, &gpair_, obj_.get());
+    monitor_.Stop("UpdateOneIter");
   }
 
-  void BoostOneIter(int iter,
-                    DMatrix* train,
-                    std::vector<bst_gpair>* in_gpair) override {
-    if (tparam.seed_per_iteration || rabit::IsDistributed()) {
-      common::GlobalRandom().seed(tparam.seed * kRandSeedMagic + iter);
+  void BoostOneIter(int iter, DMatrix* train,
+                    HostDeviceVector<GradientPair>* in_gpair) override {
+    monitor_.Start("BoostOneIter");
+    if (tparam_.seed_per_iteration || rabit::IsDistributed()) {
+      common::GlobalRandom().seed(tparam_.seed * kRandSeedMagic + iter);
     }
     this->LazyInitDMatrix(train);
     gbm_->DoBoost(train, in_gpair);
+    monitor_.Stop("BoostOneIter");
   }
 
-  std::string EvalOneIter(int iter,
-                          const std::vector<DMatrix*>& data_sets,
+  std::string EvalOneIter(int iter, const std::vector<DMatrix*>& data_sets,
                           const std::vector<std::string>& data_names) override {
+    monitor_.Start("EvalOneIter");
     std::ostringstream os;
-    os << '[' << iter << ']'
-       << std::setiosflags(std::ios::fixed);
+    os << '[' << iter << ']' << std::setiosflags(std::ios::fixed);
     if (metrics_.size() == 0) {
       metrics_.emplace_back(Metric::Create(obj_->DefaultEvalMetric()));
     }
@@ -319,15 +395,17 @@ class LearnerImpl : public Learner {
       obj_->EvalTransform(&preds_);
       for (auto& ev : metrics_) {
         os << '\t' << data_names[i] << '-' << ev->Name() << ':'
-           << ev->Eval(preds_, data_sets[i]->info(), tparam.dsplit == 2);
+           << ev->Eval(preds_.HostVector(), data_sets[i]->Info(), tparam_.dsplit == 2);
       }
     }
+
+    monitor_.Stop("EvalOneIter");
     return os.str();
   }
 
   void SetAttr(const std::string& key, const std::string& value) override {
     attributes_[key] = value;
-    mparam.contain_extra_attrs = 1;
+    mparam_.contain_extra_attrs = 1;
   }
 
   bool GetAttr(const std::string& key, std::string* out) const override {
@@ -353,21 +431,27 @@ class LearnerImpl : public Learner {
     return out;
   }
 
-  std::pair<std::string, bst_float> Evaluate(DMatrix* data, std::string metric) {
+  std::pair<std::string, bst_float> Evaluate(DMatrix* data,
+                                             std::string metric) {
     if (metric == "auto") metric = obj_->DefaultEvalMetric();
     std::unique_ptr<Metric> ev(Metric::Create(metric.c_str()));
     this->PredictRaw(data, &preds_);
     obj_->EvalTransform(&preds_);
-    return std::make_pair(metric, ev->Eval(preds_, data->info(), tparam.dsplit == 2));
+    return std::make_pair(metric,
+                          ev->Eval(preds_.HostVector(), data->Info(), tparam_.dsplit == 2));
   }
 
-  void Predict(DMatrix* data,
-               bool output_margin,
-               std::vector<bst_float> *out_preds,
-               unsigned ntree_limit,
-               bool pred_leaf) const override {
-    if (pred_leaf) {
-      gbm_->PredictLeaf(data, out_preds, ntree_limit);
+  void Predict(DMatrix* data, bool output_margin,
+               HostDeviceVector<bst_float>* out_preds, unsigned ntree_limit,
+               bool pred_leaf, bool pred_contribs, bool approx_contribs,
+               bool pred_interactions) const override {
+    if (pred_contribs) {
+      gbm_->PredictContribution(data, &out_preds->HostVector(), ntree_limit, approx_contribs);
+    } else if (pred_interactions) {
+      gbm_->PredictInteractionContributions(data, &out_preds->HostVector(), ntree_limit,
+                                            approx_contribs);
+    } else if (pred_leaf) {
+      gbm_->PredictLeaf(data, &out_preds->HostVector(), ntree_limit);
     } else {
       this->PredictRaw(data, out_preds, ntree_limit);
       if (!output_margin) {
@@ -379,77 +463,81 @@ class LearnerImpl : public Learner {
  protected:
   // check if p_train is ready to used by training.
   // if not, initialize the column access.
-  inline void LazyInitDMatrix(DMatrix *p_train) {
-    if (!p_train->HaveColAccess()) {
-      int ncol = static_cast<int>(p_train->info().num_col);
+  inline void LazyInitDMatrix(DMatrix* p_train) {
+    if (tparam_.tree_method == 3 || tparam_.tree_method == 4 ||
+        tparam_.tree_method == 5 || name_gbm_ == "gblinear") {
+      return;
+    }
+
+    monitor_.Start("LazyInitDMatrix");
+    if (!p_train->HaveColAccess(true)) {
+      auto ncol = static_cast<int>(p_train->Info().num_col_);
       std::vector<bool> enabled(ncol, true);
       // set max row per batch to limited value
       // in distributed mode, use safe choice otherwise
-      size_t max_row_perbatch = tparam.max_row_perbatch;
-      const size_t safe_max_row = static_cast<size_t>(32UL << 10UL);
+      size_t max_row_perbatch = tparam_.max_row_perbatch;
+      const auto safe_max_row = static_cast<size_t>(32ul << 10ul);
 
-      if (tparam.tree_method == 0 &&
-          p_train->info().num_row >= (4UL << 20UL)) {
-        LOG(CONSOLE) << "Tree method is automatically selected to be \'approx\'"
-                     << " for faster speed."
-                     << " to use old behavior(exact greedy algorithm on single machine),"
-                     << " set tree_method to \'exact\'";
+      if (tparam_.tree_method == 0 && p_train->Info().num_row_ >= (4UL << 20UL)) {
+        LOG(CONSOLE)
+            << "Tree method is automatically selected to be \'approx\'"
+            << " for faster speed."
+            << " to use old behavior(exact greedy algorithm on single machine),"
+            << " set tree_method to \'exact\'";
         max_row_perbatch = std::min(max_row_perbatch, safe_max_row);
       }
 
-      if (tparam.tree_method == 1) {
+      if (tparam_.tree_method == 1) {
         LOG(CONSOLE) << "Tree method is selected to be \'approx\'";
         max_row_perbatch = std::min(max_row_perbatch, safe_max_row);
       }
 
-      if (tparam.test_flag == "block" || tparam.dsplit == 2) {
+      if (tparam_.test_flag == "block" || tparam_.dsplit == 2) {
         max_row_perbatch = std::min(max_row_perbatch, safe_max_row);
       }
       // initialize column access
-      p_train->InitColAccess(enabled,
-                             tparam.prob_buffer_row,
-                             max_row_perbatch);
+      p_train->InitColAccess(enabled, tparam_.prob_buffer_row, max_row_perbatch, true);
     }
 
     if (!p_train->SingleColBlock() && cfg_.count("updater") == 0) {
-      if (tparam.tree_method == 2) {
+      if (tparam_.tree_method == 2) {
         LOG(CONSOLE) << "tree method is set to be 'exact',"
-                     << " but currently we are only able to proceed with approximate algorithm";
+                     << " but currently we are only able to proceed with "
+                        "approximate algorithm";
       }
       cfg_["updater"] = "grow_histmaker,prune";
-      if (gbm_.get() != nullptr) {
+      if (gbm_ != nullptr) {
         gbm_->Configure(cfg_.begin(), cfg_.end());
       }
     }
+    monitor_.Stop("LazyInitDMatrix");
   }
 
   // return whether model is already initialized.
-  inline bool ModelInitialized() const {
-    return gbm_.get() != nullptr;
-  }
+  inline bool ModelInitialized() const { return gbm_ != nullptr; }
   // lazily initialize the model if it haven't yet been initialized.
   inline void LazyInitModel() {
     if (this->ModelInitialized()) return;
     // estimate feature bound
     unsigned num_feature = 0;
-    for (size_t i = 0; i < cache_.size(); ++i) {
-      CHECK(cache_[i] != nullptr);
+    for (auto & matrix : cache_) {
+      CHECK(matrix != nullptr);
       num_feature = std::max(num_feature,
-                             static_cast<unsigned>(cache_[i]->info().num_col));
+                             static_cast<unsigned>(matrix->Info().num_col_));
     }
     // run allreduce on num_feature to find the maximum value
     rabit::Allreduce<rabit::op::Max>(&num_feature, 1);
-    if (num_feature > mparam.num_feature) {
-      mparam.num_feature = num_feature;
+    if (num_feature > mparam_.num_feature) {
+      mparam_.num_feature = num_feature;
     }
     // setup
-    cfg_["num_feature"] = common::ToString(mparam.num_feature);
-    CHECK(obj_.get() == nullptr && gbm_.get() == nullptr);
+    cfg_["num_feature"] = common::ToString(mparam_.num_feature);
+    CHECK(obj_ == nullptr && gbm_ == nullptr);
     obj_.reset(ObjFunction::Create(name_obj_));
     obj_->Configure(cfg_.begin(), cfg_.end());
     // reset the base score
-    mparam.base_score = obj_->ProbToMargin(mparam.base_score);
-    gbm_.reset(GradientBooster::Create(name_gbm_, cache_, mparam.base_score));
+    mparam_.base_score = obj_->ProbToMargin(mparam_.base_score);
+    gbm_.reset(GradientBooster::Create(name_gbm_, cache_, mparam_.base_score));
     gbm_->Configure(cfg_.begin(), cfg_.end());
   }
   /*!
@@ -459,19 +547,17 @@ class LearnerImpl : public Learner {
    * \param ntree_limit limit number of trees used for boosted tree
    *   predictor, when it equals 0, this means we are using all the trees
    */
-  inline void PredictRaw(DMatrix* data,
-                         std::vector<bst_float>* out_preds,
+  inline void PredictRaw(DMatrix* data, HostDeviceVector<bst_float>* out_preds,
                          unsigned ntree_limit = 0) const {
-    CHECK(gbm_.get() != nullptr)
+    CHECK(gbm_ != nullptr)
         << "Predict must happen after Load or InitModel";
-    gbm_->Predict(data,
-                  out_preds,
-                  ntree_limit);
+    gbm_->PredictBatch(data, out_preds, ntree_limit);
   }
+
   // model parameter
-  LearnerModelParam mparam;
+  LearnerModelParam mparam_;
   // training parameter
-  LearnerTrainParam tparam;
+  LearnerTrainParam tparam_;
   // configurations
   std::map<std::string, std::string> cfg_;
   // attributes
@@ -481,18 +567,21 @@ class LearnerImpl : public Learner {
   // name of objective function
   std::string name_obj_;
   // temporal storages for prediction
-  std::vector<bst_float> preds_;
+  HostDeviceVector<bst_float> preds_;
   // gradient pairs
-  std::vector<bst_gpair> gpair_;
+  HostDeviceVector<GradientPair> gpair_;
 
  private:
   /*! \brief random number transformation seed. */
   static const int kRandSeedMagic = 127;
   // internal cached dmatrix
   std::vector<std::shared_ptr<DMatrix> > cache_;
+
+  common::Monitor monitor_;
 };
 
-Learner* Learner::Create(const std::vector<std::shared_ptr<DMatrix> >& cache_data) {
+Learner* Learner::Create(
+    const std::vector<std::shared_ptr<DMatrix> >& cache_data) {
   return new LearnerImpl(cache_data);
 }
 }  // namespace xgboost
